/* rawallpass - rawallpass all elements of the input and put out as a "constant" on output */

#include "m_pd.h"
#include <math.h>

typedef struct _rawallpass
{
    t_object x_obj;
    t_float x_f;
    t_float x_state;
} t_rawallpass;

static t_class *rawallpass_class;

static void *rawallpass_new(void)
{
    t_rawallpass *x = (t_rawallpass *)pd_new(rawallpass_class);
    inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
    outlet_new(&x->x_obj, gensym("signal"));
    x->x_f = x->x_state = 0;
    return (x);
}

static t_int *rawallpass_perform(t_int *w)
{
    t_rawallpass *x = (t_rawallpass *)(w[1]);
    t_float *in1 = (t_float *)(w[2]);
    t_float *in2 = (t_float *)(w[3]);
    t_float *out = (t_float *)(w[4]);
    int n = (int)(w[5]), i;
    float state = x->x_state;
    if (PD_BIGORSMALL(state))
        state = 0;
    for (i = 0; i < n; i++)
    {
        float c = *in2++;
        float z = 1 - c*c;
        float s = sqrt(z < 0 ? 0 : z);
        float f = *in1++;
        float newstate = s * f + c * state;
        *out++ = c * f - s * state;
        state = newstate;
    }
    x->x_state = state;
    return (w+6);
}

static void rawallpass_dsp(t_rawallpass *x, t_signal **sp)
{
    dsp_add(rawallpass_perform, 5, x, sp[0]->s_vec, sp[1]->s_vec,
        sp[2]->s_vec, sp[0]->s_n);
}

void rawallpass_tilde_setup(void)
{
    int i;
    rawallpass_class = class_new(gensym("rawallpass~"),
        (t_newmethod)rawallpass_new, 0, sizeof(t_rawallpass), 0, 0);
    class_addmethod(rawallpass_class, (t_method)rawallpass_dsp, gensym("dsp"), A_CANT, 0);
    class_addmethod(rawallpass_class, (t_method)rawallpass_dsp, gensym("signal"),
    	A_CANT, 0);
    CLASS_MAINSIGNALIN(rawallpass_class, t_rawallpass, x_f);
}
